@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -27,7 +27,6 @@
 #include <glib.h>
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
 #endif /* !TOTEM_PL_PARSER_MINI */
@@ -40,11 +39,10 @@
 gboolean
 totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 			   TotemPlParserIterFunc func, 
-			   const char *output, const char *title,
+			   GFile *output, const char *title,
 			   gpointer user_data, GError **error)
 {
-	GnomeVFSHandle *handle;
-	GnomeVFSResult res;
+	GFileOutputStream *stream;
 	int num_entries_total, num_entries, i;
 	char *buf;
 	gboolean success;
@@ -52,53 +50,34 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 	num_entries = totem_pl_parser_num_entries (parser, model, func, user_data);
 	num_entries_total = gtk_tree_model_iter_n_children (model, NULL);
 
-	res = gnome_vfs_open (&handle, output, GNOME_VFS_OPEN_WRITE);
-	if (res == GNOME_VFS_ERROR_NOT_FOUND) {
-		res = gnome_vfs_create (&handle, output,
-				GNOME_VFS_OPEN_WRITE, FALSE,
-				GNOME_VFS_PERM_USER_WRITE
-				| GNOME_VFS_PERM_USER_READ
-				| GNOME_VFS_PERM_GROUP_READ);
-	}
-
-	if (res != GNOME_VFS_OK) {
-		g_set_error(error,
-			    TOTEM_PL_PARSER_ERROR,
-			    TOTEM_PL_PARSER_ERROR_VFS_OPEN,
-			    _("Couldn't open file '%s': %s"),
-			    output, gnome_vfs_result_to_string (res));
+	stream = g_file_replace (output, NULL, FALSE, G_FILE_CREATE_NONE, NULL, error);
+	if (stream == NULL)
 		return FALSE;
-	}
 
 	buf = g_strdup ("[playlist]\n");
-	success = totem_pl_parser_write_string (handle, buf, error);
+	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 	g_free (buf);
 	if (success == FALSE)
 		return FALSE;
 
 	if (title != NULL) {
 		buf = g_strdup_printf ("X-GNOME-Title=%s\n", title);
-		success = totem_pl_parser_write_string (handle, buf, error);
+		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (buf);
 		if (success == FALSE)
-		{
-			gnome_vfs_close (handle);
 			return FALSE;
-		}
 	}
 
 	buf = g_strdup_printf ("NumberOfEntries=%d\n", num_entries);
-	success = totem_pl_parser_write_string (handle, buf, error);
+	success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 	g_free (buf);
 	if (success == FALSE)
-	{
-		gnome_vfs_close (handle);
 		return FALSE;
-	}
 
 	for (i = 1; i <= num_entries_total; i++) {
 		GtkTreeIter iter;
 		char *url, *title, *relative;
+		GFile *file;
 		gboolean custom_title;
 
 		if (gtk_tree_model_iter_nth_child (model, &iter, NULL, i - 1) == FALSE)
@@ -106,23 +85,22 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 
 		func (model, &iter, &url, &title, &custom_title, user_data);
 
-		if (totem_pl_parser_scheme_is_ignored (parser, url) != FALSE)
-		{
+		file = g_file_new_for_uri (url);
+		if (totem_pl_parser_scheme_is_ignored (parser, file) != FALSE) {
 			g_free (url);
 			g_free (title);
+			g_object_unref (file);
 			continue;
 		}
+		g_object_unref (file);
 
-		relative = totem_pl_parser_relative (url, output);
-		buf = g_strdup_printf ("File%d=%s\n", i,
-				relative ? relative : url);
+		relative = totem_pl_parser_relative (output, url);
+		buf = g_strdup_printf ("File%d=%s\n", i, relative ? relative : url);
 		g_free (relative);
 		g_free (url);
-		success = totem_pl_parser_write_string (handle, buf, error);
+		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (buf);
-		if (success == FALSE)
-		{
-			gnome_vfs_close (handle);
+		if (success == FALSE) {
 			g_free (title);
 			return FALSE;
 		}
@@ -133,25 +111,25 @@ totem_pl_parser_write_pls (TotemPlParser *parser, GtkTreeModel *model,
 		}
 
 		buf = g_strdup_printf ("Title%d=%s\n", i, title);
-		success = totem_pl_parser_write_string (handle, buf, error);
+		success = totem_pl_parser_write_string (G_OUTPUT_STREAM (stream), buf, error);
 		g_free (buf);
 		g_free (title);
 		if (success == FALSE)
-		{
-			gnome_vfs_close (handle);
 			return FALSE;
-		}
 	}
 
-	gnome_vfs_close (handle);
+	g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
 	return TRUE;
 }
 
 TotemPlParserResult
-totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
-				       const char *base, const char *contents)
+totem_pl_parser_add_pls_with_contents (TotemPlParser *parser,
+				       GFile *file,
+				       GFile *_base_file,
+				       const char *contents)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	GFile *base_file;
 	char **lines;
 	int i, num_entries;
 	char *split_char, *playlist_title;
@@ -187,7 +165,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
 	if (playlist_title != NULL) {
 		totem_pl_parser_add_url (parser,
 					 TOTEM_PL_PARSER_FIELD_IS_PLAYLIST, TRUE,
-					 TOTEM_PL_PARSER_FIELD_URL, url,
+					 TOTEM_PL_PARSER_FIELD_FILE, file,
 					 TOTEM_PL_PARSER_FIELD_TITLE, playlist_title,
 					 NULL);
 	}
@@ -210,10 +188,16 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
 			goto bail;
 	}
 
+	/* Base? */
+	if (_base_file == NULL)
+		base_file = g_file_get_parent (file);
+	else
+		base_file = g_object_ref (_base_file);
+
 	retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
 	for (i = 1; i <= num_entries; i++) {
-		char *file, *title, *genre, *length;
+		char *file_str, *title, *genre, *length;
 		char *file_key, *title_key, *genre_key, *length_key;
 		gint64 length_num;
 
@@ -224,7 +208,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
 		/* Genre is our own little extension */
 		genre_key = g_strdup_printf ("genre%d", i);
 
-		file = totem_pl_parser_read_ini_line_string (lines, (const char*)file_key, dos_mode);
+		file_str = totem_pl_parser_read_ini_line_string (lines, (const char*)file_key, dos_mode);
 		title = totem_pl_parser_read_ini_line_string (lines, (const char*)title_key, dos_mode);
 		genre = totem_pl_parser_read_ini_line_string (lines, (const char*)genre_key, dos_mode);
 		length = totem_pl_parser_read_ini_line_string (lines, (const char*)length_key, dos_mode);
@@ -234,8 +218,7 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
 		g_free (genre_key);
 		g_free (length_key);
 
-		if (file == NULL)
-		{
+		if (file_str == NULL) {
 			g_free (title);
 			g_free (genre);
 			g_free (length);
@@ -251,41 +234,39 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
 		if (length != NULL)
 			length_num = totem_pl_parser_parse_duration (length, parser->priv->debug);
 
-		if (strstr (file, "://") != NULL || file[0] == G_DIR_SEPARATOR) {
-			if (length_num < 0 || totem_pl_parser_parse_internal (parser, file, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
+		if (strstr (file_str, "://") != NULL || file_str[0] == G_DIR_SEPARATOR) {
+			GFile *target;
+
+			target = g_file_new_for_commandline_arg (file_str);
+			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, NULL) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 				totem_pl_parser_add_url (parser,
-							 TOTEM_PL_PARSER_FIELD_URL, file,
+							 TOTEM_PL_PARSER_FIELD_URL, file_str,
 							 TOTEM_PL_PARSER_FIELD_TITLE, title,
 							 TOTEM_PL_PARSER_FIELD_GENRE, genre,
 							 TOTEM_PL_PARSER_FIELD_DURATION, length,
-							 TOTEM_PL_PARSER_FIELD_BASE, base, NULL);
+							 TOTEM_PL_PARSER_FIELD_BASE_FILE, base_file, NULL);
 			}
+			g_object_unref (target);
 		} else {
-			char *base;
+			GFile *target;
 
-			/* Try with a base */
-			base = totem_pl_parser_base_url (url);
+			target = g_file_get_child_for_display_name (base_file, file_str, NULL);
 
-			if (length_num < 0 || totem_pl_parser_parse_internal (parser, file, base) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
-				char *escaped, *uri;
+			if (length_num < 0 || totem_pl_parser_parse_internal (parser, target, base_file) != TOTEM_PL_PARSER_RESULT_SUCCESS) {
 
-				escaped = gnome_vfs_escape_path_string (file);
-				uri = g_strdup_printf ("%s/%s", base, escaped);
-				g_free (escaped);
 				totem_pl_parser_add_url (parser,
-							 TOTEM_PL_PARSER_FIELD_URL, uri,
+							 TOTEM_PL_PARSER_FIELD_FILE, target,
 							 TOTEM_PL_PARSER_FIELD_TITLE, title,
 							 TOTEM_PL_PARSER_FIELD_GENRE, genre,
 							 TOTEM_PL_PARSER_FIELD_DURATION, length,
-							 TOTEM_PL_PARSER_FIELD_BASE, base, NULL);
-				g_free (uri);
+							 TOTEM_PL_PARSER_FIELD_BASE_FILE, base_file, NULL);
 			}
 
-			g_free (base);
+			g_object_unref (target);
 		}
 
 		parser->priv->fallback = fallback;
-		g_free (file);
+		g_free (file_str);
 		g_free (title);
 		g_free (genre);
 		g_free (length);
@@ -294,6 +275,8 @@ totem_pl_parser_add_pls_with_contents (TotemPlParser *parser, const char *url,
 	if (playlist_title != NULL)
 		totem_pl_parser_playlist_end (parser, playlist_title);
 
+	g_object_unref (base_file);
+
 bail:
 	g_free (playlist_title);
 	g_strfreev (lines);
@@ -302,14 +285,16 @@ bail:
 }
 
 TotemPlParserResult
-totem_pl_parser_add_pls (TotemPlParser *parser, const char *url,
-			 const char *base, gpointer data)
+totem_pl_parser_add_pls (TotemPlParser *parser,
+			 GFile *file,
+			 GFile *base_file,
+			 gpointer data)
 {
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	char *contents;
-	int size;
+	gsize size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	if (size == 0) {
@@ -317,7 +302,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url,
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
 
-	retval = totem_pl_parser_add_pls_with_contents (parser, url, base, contents);
+	retval = totem_pl_parser_add_pls_with_contents (parser, file, base_file, contents);
 	g_free (contents);
 
 	return retval;
