@@ -97,6 +97,7 @@
 
 #include <string.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gi18n-lib.h>
 #include <gio/gio.h>
 
@@ -198,6 +199,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("video/mp4", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktimeplayer", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/xml", totem_pl_parser_add_xml_feed, totem_pl_parser_is_xml_feed),
@@ -353,8 +355,8 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (TotemPlParserClass, entry_parsed),
 			      NULL, NULL,
-			      totemplparser_marshal_VOID__STRING_POINTER,
-			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
+			      totemplparser_marshal_VOID__STRING_BOXED,
+			      G_TYPE_NONE, 2, G_TYPE_STRING, TOTEM_TYPE_PL_PARSER_METADATA);
 	/**
 	 * TotemPlParser::playlist-started:
 	 * @parser: the object which received the signal
@@ -373,8 +375,8 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (TotemPlParserClass, playlist_started),
 			      NULL, NULL,
-			      totemplparser_marshal_VOID__STRING_POINTER,
-			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
+			      totemplparser_marshal_VOID__STRING_BOXED,
+			      G_TYPE_NONE, 2, G_TYPE_STRING, TOTEM_TYPE_PL_PARSER_METADATA);
 	/**
 	 * TotemPlParser::playlist-ended:
 	 * @parser: the object which received the signal
@@ -495,6 +497,10 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "Object representing the GFile for an entry", G_TYPE_FILE,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_object ("gfile-object-base", "gfile-object-base",
+				     "Object representing the GFile for base URI of an entry", G_TYPE_FILE,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("download-url", "download-url",
 				     "String representing the location of a download URL", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
@@ -639,16 +645,16 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 
 	/* Stat for a block device, we're screwed as far as speed
 	 * is concerned now */
-	if (g_file_has_uri_scheme (file, "file") != FALSE) {
-		GFileInfo *info;
-		info = g_file_query_info (file, G_FILE_ATTRIBUTE_UNIX_DEVICE,
-					  G_FILE_QUERY_INFO_NONE, NULL, NULL);
-		if (info != NULL && g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_UNIX_DEVICE)) {
-			g_object_unref (info);
+	if (g_file_is_native (file) != FALSE) {
+		struct stat buf;
+		char *path;
+
+		path = g_file_get_path (file);
+		if (g_stat (path, &buf) == 0 && S_ISBLK (buf.st_mode)) {
+			g_free (path);
 			return g_strdup (BLOCK_DEVICE_TYPE);
 		}
-		if (info != NULL)
-			g_object_unref (info);
+		g_free (path);
 	}
 
 	/* Open the file. */
@@ -667,7 +673,7 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 	/* Read the whole thing, up to MIME_READ_CHUNK_SIZE */
 	buffer = g_malloc (MIME_READ_CHUNK_SIZE);
 	bytes_read = g_input_stream_read (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, NULL, &error);
-	g_input_stream_close (G_INPUT_STREAM (stream), NULL, NULL);
+	g_object_unref (G_INPUT_STREAM (stream));
 	if (bytes_read == -1) {
 		g_free (buffer);
 		return NULL;
@@ -754,7 +760,7 @@ totem_pl_parser_write_string (GOutputStream *stream, const char *buf, GError **e
 
 /**
  * totem_pl_parser_write_buffer:
- * @handle: a #GnomeVFSHandle to an open file
+ * @stream: a #GFileOutputStream to an open file
  * @buf: the string buffer to write out
  * @len: the length of the string to write out
  * @error: return location for a #GError, or %NULL
@@ -772,7 +778,7 @@ totem_pl_parser_write_buffer (GOutputStream *stream, const char *buf, guint len,
 				       buf, len,
 				       &bytes_written,
 				       NULL, error) == FALSE) {
-		g_output_stream_close (stream, NULL, NULL);
+		g_object_unref (stream);
 		return FALSE;
 	}
 
@@ -1189,7 +1195,7 @@ totem_pl_parser_add_url_valist (TotemPlParser *parser,
 		}
 	}
 
-	g_hash_table_destroy (metadata);
+	g_hash_table_unref (metadata);
 
 	g_free (url);
 	g_object_unref (G_OBJECT (parser));
@@ -1353,6 +1359,28 @@ totem_pl_parser_ignore (TotemPlParser *parser, const char *url)
 	return TRUE;
 }
 
+/**
+ * totem_pl_parser_cleanup_xml:
+ * @contents: the contents of the file
+ *
+ * Removes HTML comments from a string representing the contents of an XML file.
+ * The function modifies the string in place.
+ */
+void
+totem_pl_parser_cleanup_xml (char *contents)
+{
+	char *needle;
+
+	while ((needle = strstr (contents, "<!--")) != NULL) {
+		while (strncmp (needle, "-->", 3) != 0) {
+			*needle = ' ';
+			needle++;
+			if (*needle == '\0')
+				break;
+		}
+	}
+}
+
 static gboolean
 totem_pl_parser_ignore_from_mimetype (TotemPlParser *parser, const char *mimetype)
 {
@@ -1387,17 +1415,25 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 	if (g_file_has_uri_scheme (file, "mms") != FALSE
 			|| g_file_has_uri_scheme (file, "rtsp") != FALSE
-			|| g_file_has_uri_scheme (file, "icy") != FALSE) {
-		DEBUG(file, g_print ("URL '%s' is MMS, RTSP or ICY, ignoring\n", uri));
+			|| g_file_has_uri_scheme (file, "icy") != FALSE
+			|| g_file_has_uri_scheme (file, "pnm") != FALSE) {
+		DEBUG(file, g_print ("URL '%s' is MMS, RTSP, PNM or ICY, not a playlist\n", uri));
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
 
 	/* Fix up itpc, see http://www.apple.com/itunes/store/podcaststechspecs.html,
-	 * as well as feed:// as used by Firefox 3 */
-	if (g_file_has_uri_scheme (file, "itpc") != FALSE || g_file_has_uri_scheme (file, "feed") != FALSE) {
-		DEBUG(file, g_print ("URL '%s' is getting special cased for ITPC/FEED parsing\n", uri));
+	 * feed:// as used by Firefox 3,
+	 * as well as zcast:// as used by ZENCast */
+	if (g_file_has_uri_scheme (file, "itpc") != FALSE
+	    || g_file_has_uri_scheme (file, "feed") != FALSE
+	    || g_file_has_uri_scheme (file, "zcast") != FALSE) {
+		DEBUG(file, g_print ("URL '%s' is getting special cased for ITPC/FEED/ZCAST parsing\n", uri));
 		return totem_pl_parser_add_itpc (parser, file, base_file, NULL);
 	}
+	if (g_file_has_uri_scheme (file, "zune") != FALSE) {
+		DEBUG(file, g_print ("URL '%s' is getting special cased for ZUNE parsing\n", uri));
+		return totem_pl_parser_add_zune (parser, file, base_file, NULL);
+	}
 	/* Try itms Podcast references, see itunes.py in PenguinTV */
 	if (totem_pl_parser_is_itms_feed (file) != FALSE) {
 	    	DEBUG(file, g_print ("URL '%s' is getting special cased for ITMS parsing\n", uri));
@@ -1429,7 +1465,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	}
 
 	DEBUG(file, g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", uri, mimetype));
-	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0) {
+	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0
+	    || (g_file_is_native (file) && g_content_type_is_a (mimetype, "text/plain") != FALSE)) {
 		g_free (mimetype);
 		mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
 		DEBUG(file, g_print ("_get_mime_type_with_data for '%s' returned '%s'\n", uri, mimetype ? mimetype : "NULL"));
@@ -1476,7 +1513,16 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 					g_free (data);
 					return TOTEM_PL_PARSER_RESULT_IGNORED;
 				}
+				if (base_file == NULL)
+					base_file = g_file_get_parent (file);
+				else
+					base_file = g_object_ref (base_file);
+
+				DEBUG (file, g_print ("Using %s function for '%s'\n", special_types[i].mimetype, uri));
 				ret = (* special_types[i].func) (parser, file, base_file, data);
+
+				g_object_unref (base_file);
+
 				found = TRUE;
 				break;
 			}
@@ -1495,7 +1541,15 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 						break;
 					}
 				}
-				ret = (* dual_types[i].func) (parser, file, base_file, data);
+				if (base_file == NULL)
+					base_file = g_file_get_parent (file);
+				else
+					base_file = g_object_ref (base_file);
+
+				ret = (* dual_types[i].func) (parser, file, base_file ? base_file : file, data);
+
+				g_object_unref (base_file);
+
 				found = TRUE;
 				break;
 			}
@@ -1745,7 +1799,9 @@ totem_pl_parser_mime_type_from_data (gconstpointer data, int len)
 #endif
 
 	if (mime_type != NULL &&
-	    (strcmp (mime_type, "text/plain") == 0 || strcmp (mime_type, "application/octet-stream") == 0)) {
+	    (strcmp (mime_type, "text/plain") == 0 ||
+	     strcmp (mime_type, "application/octet-stream") == 0 ||
+	     strcmp (mime_type, "application/xml") == 0)) {
 		PlaylistIdenCallback func;
 		guint i;
 
@@ -1859,3 +1915,19 @@ totem_pl_parser_can_parse_from_filename (const char *filename, gboolean debug)
 	return retval;
 }
 
+
+GType
+totem_pl_parser_metadata_get_type (void)
+{
+	static volatile gsize g_define_type_id__volatile = 0;
+	if (g_once_init_enter (&g_define_type_id__volatile))
+	{ 
+		GType g_define_type_id = g_boxed_type_register_static (
+		    g_intern_static_string ("TotemPlParserMetadata"),
+		    (GBoxedCopyFunc) g_hash_table_ref,
+		    (GBoxedFreeFunc) g_hash_table_unref);
+		g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
+	}
+	return g_define_type_id__volatile;
+}
+
